
/******************************************************************************
 *
 * List.C
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Headers
 *
 ******************************************************************************/

#include <stdio.h>

#include "intrinsic.h"
#include "list.h"
#include "mem.h"

/******************************************************************************
 *
 * Local Constants and Macros
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Declarations
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Prototypes
 *
 ******************************************************************************/

/******************************************************************************
 *
 * ListAlloc
 *
 ******************************************************************************/

#ifdef VMS
extern List ListAlloc (void)
#else
extern List ListAlloc ()
#endif
{
  List list;

  /* allocate */
  list = (List) MemAlloc (sizeof(_List), Initialize);

  /* return */
  return (list);
}

/******************************************************************************
 *
 * ListDealloc
 *
 ******************************************************************************/

#ifdef VMS
extern Status ListDealloc (List list)
#else
extern Status ListDealloc (list)

  List list;
#endif

{
  /* free */
  MemDealloc (list);

  /* done */
  return (Success);
}

/******************************************************************************
 *
 * ListNodeAlloc
 *
 ******************************************************************************/

#ifdef VMS
extern ListNode ListNodeAlloc (void)
#else
extern ListNode ListNodeAlloc ()
#endif

{
  ListNode node;


  /* allocate a node */
  node = (ListNode) MemAlloc (sizeof(_ListNode), NoInitialize);

  /* done */
  return (node);
}

/******************************************************************************
 *
 * ListNodeDealloc
 *
 ******************************************************************************/

#ifdef VMS
extern Status ListNodeDealloc (ListNode node)
#else
extern Status ListNodeDealloc (node)

  ListNode node;
#endif

{
  /* free */
  MemDealloc (node);

  /* done */
  return (Success);
}

/******************************************************************************
 *
 * ListNodesDealloc
 *
 ******************************************************************************/

#ifdef VMS
extern Status ListNodesDealloc (List list)
#else
extern Status ListNodesDealloc (list)

  List list;
#endif

{
  /* TBD */

  /* free list nodes */

  /* done */
  return (Failure);
}

/******************************************************************************
 *
 * ListInit
 *
 ******************************************************************************/

#ifdef VMS
extern Status ListInit (List list)
#else
extern Status ListInit (list)

  List list;
#endif

{
  /* initialize */
  list->head = NULL;
  list->tail = NULL;

  /* done */
  return (Success);
}

/******************************************************************************
 *
 * ListAppend
 *
 ******************************************************************************/

#ifdef VMS
extern ListNode ListAppend (List list, Pointer data)
#else
extern ListNode ListAppend (list, data)

  List    list;
  Pointer data;
#endif

{
  ListNode node;

  /* allocate a node */
  node = (ListNode) MemAlloc (sizeof(_ListNode), NoInitialize);

  /* save data */
  node->data = data;

  /* append to list */
  if (! list->head) {
    node->previous = NULL;
    node->next     = NULL;
    list->head     = node;
    list->tail     = node;
  }
  else {
    node->previous   = list->tail;
    node->next       = NULL;
    list->tail->next = node;
    list->tail       = node;
  }

  /* done */
  return (node);
}

/******************************************************************************
 *
 * ListPreface
 *
 ******************************************************************************/

#ifdef VMS
extern ListNode ListPreface (List list, Pointer data)
#else
extern ListNode ListPreface (list, data)

  List    list;
  Pointer data;
#endif

{
  ListNode node;

  /* allocate a node */
  node = (ListNode) MemAlloc (sizeof(_ListNode), NoInitialize);

  /* save data */
  node->data = data;

  /* append to list */
  if (! list->head) {
    node->previous = NULL;
    node->next     = NULL;
    list->head     = node;
    list->tail     = node;
  }
  else {
    node->previous       = NULL;
    node->next           = list->head;
    list->head->previous = node;
    list->head           = node;
  }

  /* done */
  return (node);
}
