
/******************************************************************************
 *
 * Mem.C
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Headers
 *
 ******************************************************************************/

#include <stdio.h>

#include "intrinsic.h"
#include "mem.h"

/******************************************************************************
 *
 * Local Constants and Macros
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Declarations
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Local Prototypes
 *
 ******************************************************************************/

/******************************************************************************
 *
 * MemAlloc
 *
 ******************************************************************************/

#ifdef VMS
extern Pointer MemAlloc (int size, int initialize)
#else
extern Pointer MemAlloc (size, initialize)

  int size;
  int initialize;
#endif

{
  Pointer addr;

  /* allocate */
  if (initialize) addr = (Pointer) calloc (1, size);
  else            addr = (Pointer) malloc (size);

  /* return pointer */
  return (addr);
}

/******************************************************************************
 *
 * MemDealloc
 *
 ******************************************************************************/

#ifdef VMS
extern Status MemDealloc (Pointer addr)
#else
extern Status MemDealloc (addr)

  Pointer addr;
#endif

{
  /* deallocate */
  free (addr);

  /* done */
  return (Success);
}

/******************************************************************************
 *
 * MemFill
 *
 ******************************************************************************/

#ifdef VMS
extern Status MemFill (Pointer addr, int size, char data)
#else
extern Status MemFill (addr, size, data)

  Pointer addr;
  int     size;
  char    data;
#endif

{
  int    count;
  String p_addr;

  p_addr = (String) addr;

  /* fill */
  for (count = 0; count < size; count++) *(p_addr++) = data;

  /* done */
  return (Success);
}

/******************************************************************************
 *
 * MemCopy
 *
 ******************************************************************************/

#ifdef VMS
extern Status MemCopy (Pointer to, Pointer from, int size)
#else
extern Status MemCopy (to, from, size)

  Pointer to;
  Pointer from;
  int     size;
#endif

{
  String    p_to;
  String    p_from;

  p_to = (String) to;
  p_from = (String) from;

  /* copy */
  for (size; size > 0; size--) *(p_to++) = *(p_from++);

  /* done */
  return (Success);
}
